use crate::counter::*;
use lunatic::{ap::ProcessRef, spawn};
use serde::{Deserialize, Serialize};
use submillisecond_live_view::prelude::*;

#[derive(Clone, Serialize, Deserialize)]
pub struct CounterView {
    global_counter: ProcessRef<Counter>,
    current_value: i64,
}

impl LiveView for CounterView {
    type Events = (Increment, Decrement);

    fn mount(_uri: Uri, socket: Option<Socket>) -> Self {
        println!("{:?}", socket.is_some());
        let global_counter = ProcessRef::<Counter>::lookup(&"global_counter").unwrap();
        let current_count = global_counter.count();
        let mut result = Self {
            global_counter,
            current_value: current_count,
        };
        spawn!(|m: Mailbox<i64>| {
            loop {
                result.current_value = m.receive();
            }
        });
        result
    }

    fn render(&self) -> Rendered {
        html! {
            button @click=(Increment) { "Increment" }
            button @click=(Decrement) { "Decrement" }
            p { "Count is " (self.current_value) }
        }
    }
}

#[derive(Deserialize)]
pub struct Increment {}
impl LiveViewEvent<Increment> for CounterView {
    fn handle(state: &mut Self, _event: Increment) {
        state.current_value += 1;
        state.global_counter.increment()
    }
}

#[derive(Deserialize)]
pub struct Decrement {}
impl LiveViewEvent<Decrement> for CounterView {
    fn handle(state: &mut Self, _event: Decrement) {
        state.current_value -= 1;
        state.global_counter.decrement()
    }
}

pub type Notified = i64;
impl LiveViewEvent<Notified> for CounterView {
    fn handle(state: &mut Self, event: Notified) {
        state.current_value = event;
    }
}
